<?php

class TagServer {

    var $config = null;

    /**
     * @var mysqli
     */
    var $db = null;
    var $error = '';

    function __construct() {
        
    }

    function get_error() {
        return $this->error;
    }

    function set_error($message) {
        $this->error = $message;
    }

    function has_error() {
        return!empty($this->error);
    }

    function initialize() {
        $this->config = parse_ini_file('tagserver.ini');

        if (!isset($this->config['hostname'])) {
            $this->set_error('No hostname in ini file');
            return false;
        }

        if (!isset($this->config['port'])) {
            $this->set_error('No port in ini file');
            return false;
        }

        if (!isset($this->config['schema'])) {
            $this->set_error('No schema in ini file');
            return false;
        }

        if (!isset($this->config['username'])) {
            $this->set_error('No username in ini file');
            return false;
        }

        if (!isset($this->config['password'])) {
            $this->set_error('No password in ini file');
            return false;
        }

        $hostname = $this->config['hostname'];
        $port = $this->config['port'];
        $schema = $this->config['schema'];
        $username = $this->config['username'];
        $password = $this->config['password'];

        $this->db = new mysqli($hostname, $username, $password, $schema, $port);

        if (mysqli_connect_error()) {
            $this->set_error('Error connecting to database (' . mysqli_connect_errno() . ') ' . mysqli_connect_error());
            return false;
        }

        $result = $this->db->query("SHOW TABLES LIKE 'assignments'");
        if ($result) {
            $tableExists = $result->num_rows > 0;
            $result->free();

            if (!$tableExists) {
                $this->create_table();
            }

            return true;
        } else {
            $this->set_error('Unable to check for assignments table');
            return false;
        }
    }

    var $create_table_script = "CREATE TABLE `assignments` (
          `key` varchar(45) NOT NULL,
          `user` varchar(45) NOT NULL,
          `tag` varchar(45) NOT NULL,
          PRIMARY KEY (`key`,`user`,`tag`)
        ) ENGINE=MyISAM DEFAULT CHARSET=latin1";

    function create_table() {
        $result = $this->db->query($this->create_table_script);
    }

    function get_tags() {
        $result = $this->db->query('SELECT `user`, `key`, `tag` FROM assignments');
        if ($result) {
            $tags = array();
            while ($row = $result->fetch_assoc()) {
                $tags[] = $row;
            }
            $result->free();
            return result(true, 'Tags retrieved', $tags);
        } else {
            $this->set_error('Unable to query database for tags');
            return result(false, $this->get_error());
        }
    }

    function set_tag($action, $user, $key, $tag) {
        if ($action == 'add') {
            $stmt = $this->db->prepare('REPLACE INTO assignments (`user`, `key`, `tag`) VALUES (?, ?, ?)');
            if ($stmt) {
                $stmt->bind_param('sss', $user, $key, $tag);
                $stmt->execute();

                $stmt->close();
                return result(true, 'Success');
            } else {
                $this->set_error('Unable to prepare tag insert statement');
                return result(false, $this->get_error());
            }
        } else if ($action == 'remove') {
            $stmt = $this->db->prepare('DELETE FROM assignments WHERE `user` = ? AND `key` = ? AND `tag` = ?');
            if ($stmt) {
                $stmt->bind_param('sss', $user, $key, $tag);
                $stmt->execute();

                if ($stmt->affected_rows == 1) {
                    return result(true, 'Success');
                } else {
                    return result(false, 'Delete failed');
                }

                $stmt->close();
            } else {
                $this->set_error('Unable to prepare tag delete statement');
                return result(false, $this->get_error());
            }
        } else {
            $this->set_error('Invalid action');
            return result(false, $this->get_error());
        }
    }

}

function result($success, $message, $data = null) {
    return compact('success', 'message', 'data');
}

//START HERE//
$result = result(false, 'Default response');
if (!isset($_REQUEST['action'])) {
    $result = result(false, 'No action provided');
} else {
    $action = $_REQUEST['action'];

    $tagserver = new TagServer();
    if (!$tagserver->initialize()) {
        $result = result(false, $tagserver->get_error());
    } else {
        switch ($action) {
            case 'get':
                $result = $tagserver->get_tags();
                break;
            case 'add':
            case 'remove':

                if (!isset($_REQUEST['user']) || empty($_REQUEST['user'])) {
                    $result = result(false, 'No user provided');
                    break;
                }
                $user = $_REQUEST['user'];

                if (!isset($_REQUEST['key']) || empty($_REQUEST['key'])) {
                    $result = result(false, 'No key provided');
                    break;
                }
                $key = $_REQUEST['key'];

                if (!isset($_REQUEST['tag']) || empty($_REQUEST['tag'])) {
                    $result = result(false, 'No tag provided');
                    break;
                }
                $tag = $_REQUEST['tag'];

                $result = $tagserver->set_tag($action, $user, $key, $tag);
                break;
        }
    }
}

echo json_encode($result);
?>
